#ifndef __OCSP_HTTP_H
#define __OCSP_HTTP_H

#define HTTP_CONTINUE 0
#define HTTP_HEADER_TRUNCATED -1
#define HTTP_OVERFLOW -2
#define HTTP_INVALID_CONTENTLEN -3 
#define HTTP_ALLOC_ERROR -4
#define HTTP_BASEURI_ERROR -5
#define HTTP_URI_DECODE -6
#define HTTP_BASE64_DECODE -7
#define HTTP_INVALID_METHOD -8

typedef enum http_method {
    METHOD_POST,
    METHOD_GET,
    METHOD_ERROR
} http_method_t;

typedef struct http_rec {
    http_method_t method;
    int request_len;
    int content_len;

    char *http_raw_rec;
    char *http_data;
} http_rec_t;

http_method_t decode_http_method(const char *, int, int *);
char *http_expand_hex(const char *);
char *http_decode_base64(char *input, int len, int *tlen);
char *http_decode_uri(char *in_uri, int len);
http_rec_t *create_http_rec(void);
void free_http_rec(http_rec_t *rec);
int decode_http_content_len(char *input_buf, int input_len);
char *decode_http_post_data(char *input_buf, int input_len, int content_len, int *error);
http_rec_t *decode_http_post (char *input_buf, int input_len, int *error);
http_rec_t *decode_http_get (char *input_buf, int input_len, int *error);
http_rec_t *decode_http_request(char *input_buf, int input_len, int *error);
#endif
